package ex_nodecrud.model
{	
	import ex_nodecrud.ApplicationFacade;
	
	import flash.net.NetConnection;
	import flash.net.ObjectEncoding;
	import flash.net.Responder;
	
	import mx.collections.ArrayCollection;
	
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	public class NodesProxy extends Proxy implements IProxy
	{
		public static const NAME:String = "NodesProxy";
		
		// Drupal services
		private var drupal:NetConnection;
		
		//--------------------------------------------------
		// Constructor
		//--------------------------------------------------
		public function NodesProxy()
		{
			super(NAME);
			
			// Proxy data
			setData(new ArrayCollection());

			// Drupal connection
			drupal = new NetConnection();
			drupal.objectEncoding = flash.net.ObjectEncoding.AMF3;
			drupal.connect("http://ogo.heig-vd.ch/ria/cms/services/amfphp");
		}
		
		//--------------------------------------------------
		// Drupal interaction
		//--------------------------------------------------
		
		/**
		 * Obtenir de la liste de tous les noeuds du système
		 * @return 
		 * 		En cas de succès:
		 * 			- Envoi de la notification ApplicationFacade.LOADNODES_SUCCESS
		 * 			- L’ArrayCollection des nœuds est accessible par le getter nodes()
		 * 		En cas d'échec:
		 * 			- Envoi de la notification ApplicationFacade.LOADNODES_FAILED
		 */
		public function all():void
		{
			drupal.call("views.get", new Responder(onAllSuccess, onAllFailure), "NodeCRUD_all");
		}
		
		private function onAllSuccess(result:Object):void
		{
			var i:int = 0;
			this.nodes.removeAll();
			
			// Loading nodes
			for each (var drupalNode:Object in result as Array)
			{
				var node:Object = new Object();
				node.nid = drupalNode.nid;
				node.title = drupalNode.node_title;
				node.body = drupalNode.node_revisions_body;
				this.nodes.addItem(node);
			}
			
			// Notify other components
			sendNotification(ApplicationFacade.LOADNODES_SUCCESS);
		}
		
		private function onAllFailure(error:Object):void
		{
			sendNotification(ApplicationFacade.LOADNODES_FAILED, error.toString());
		}
		
		/**
		 * Créer un nouveau noeud
		 * @param Object(title, body)
		 * @return 
		 * 		En cas de succès:
		 * 			- Envoi de la notification ApplicationFacade.CREATENODES_SUCCESS
		 * 			  avec pour paramètre le noeud créé Object(nid, title, body)
		 * 		En cas d'échec:
		 * 			- Envoi de la notification ApplicationFacade.CREATENODES_FAILED
		 */
		private var createNode:Object;
		public function create(node : Object):void
		{
			node.type = "nodecrud";
			createNode = node;
			drupal.call("node.save", new Responder(onCreateSuccess, onCreateFailure), node);
		}
		
		private function onCreateSuccess(result:Object):void
		{
			createNode.nid = result;
			ApplicationFacade.getInstance().sendNotification(ApplicationFacade.CREATENODE_SUCCESS, createNode);
		}
		
		private function onCreateFailure(error:Object):void
		{
			ApplicationFacade.getInstance().sendNotification(ApplicationFacade.CREATENODE_FAILED, error.toString());
		}
		
		/**
		 * Modifier un noeud existant
		 * @param Object(nid, title, body)
		 * @return 
		 * 		En cas de succès:
		 * 			- Envoi de la notification ApplicationFacade.UPDATENODES_SUCCESS
		 * 			  avec pour paramètre le noeud modifié Object(nid, title, body)
		 * 		En cas d'échec:
		 * 			- Envoi de la notification ApplicationFacade.UPDATENODES_FAILED
		 */
		private var updateNode:Object;
		public function update(node : Object):void
		{
			updateNode = node;
			drupal.call("node.save", new Responder(onUpdateSuccess, onUpdateFailure), node);
		}
		
		private function onUpdateSuccess(result:Object):void
		{
			ApplicationFacade.getInstance().sendNotification(ApplicationFacade.UPDATENODE_SUCCESS, updateNode);
		}
		
		private function onUpdateFailure(error:Object):void
		{
			ApplicationFacade.getInstance().sendNotification(ApplicationFacade.UPDATENODE_FAILED, error.toString());
		}
		
		/**
		 * Supprimer un noeud
		 * @param Object(nid, title, body)
		 * @return 
		 * 		En cas de succès:
		 * 			- Envoi de la notification ApplicationFacade.DELETENODES_SUCCESS
		 * 			  avec pour paramètre le noeud supprimé Object(nid, title, body)
		 * 		En cas d'échec:
		 * 			- Envoi de la notification ApplicationFacade.DELETENODES_FAILED
		 */
		private var deleteNode:Object;
		public function remove(node:Object):void
		{
			deleteNode = node;
			drupal.call("node.delete", new Responder(onRemoveSuccess, onRemoveFailure), node.nid);
		}
		
		private function onRemoveSuccess(result:Object):void
		{
			ApplicationFacade.getInstance().sendNotification(ApplicationFacade.DELETENODE_SUCCESS, deleteNode);
		}
		
		private function onRemoveFailure(error:Object):void
		{
			ApplicationFacade.getInstance().sendNotification(ApplicationFacade.DELETENODE_FAILED, error.toString());
		}
		
		//--------------------------------------------------
		// Accessors
		//--------------------------------------------------
		public function get nodes():ArrayCollection
		{
			return getData() as ArrayCollection;
		}
	}
}
